@@ -108,7 +108,7 @@ class MessageHandler:
 class BluemanCollector():
     """grpc client to run blueman.proto rpc calls
     """
-    def __init__(self, interval : int, uid: str, conf : dict):
+    def __init__(self, interval : int, uid: str, conf : dict, retry_delay : int):
         """class constructor
 
         Args:
@@ -124,6 +124,8 @@ class BluemanCollector():
         self._supported_data_types = (list, dict, str, int, float, bool)
         self._installed_package_hash = ''
         self._firmware_info_hash = ''
+        self._retry_delay = retry_delay
+        self._delayed_messages = {}
 
     def __del__(self):
         """class destructor - close instance channel
@@ -157,7 +159,6 @@ class BluemanCollector():
             "cpu_frequency_health":    self._collect_cpu_frequency_health,
             "power_consumption_health": self._collect_power_consumption_health,
             "system_services_health":  self._collect_system_services_health,
-            "logs_health":             self._collect_logs_health,
             "dpu_rshim_log_health":    self._collect_dpu_rshim_log_health,
             "nvme_log_health":         self._collect_nvme_log_health,
             "kernel_modules_health":   self._collect_kernel_modules_health,
@@ -442,16 +443,6 @@ class BluemanCollector():
         response = self._stub.system_services_health(request, metadata=self._call_metadata)
         return self._get_repeated_data_dict_list(response)
 
-    def _collect_logs_health(self) -> list:
-        """collect logs health via rpc
-
-        Returns:
-            list: parsed rpc response
-        """
-        request = blueman_pb2.LogsHealthRequest()
-        response = self._stub.logs_health(request, metadata=self._call_metadata)
-        return self._get_repeated_data_dict_list(response)
-
     def _collect_dpu_rshim_log_health(self) -> list:
         """
         collect dpu rshim log via grpc
@@ -514,8 +505,7 @@ class BluemanCollector():
                 item = {
                     "name": service_data.name,
                     "container_id": service_data.container_id,
-                    "state": service_data.state,
-                    "logs": service_data.logs,
+                    "state": service_data.state
                 }
                 res.append(item)
         return res
@@ -528,11 +518,20 @@ class BluemanCollector():
         """
         data = []
         for handler in self._message_handlers:
+            name = handler.message_name
+            delayed_message_time = self._delayed_messages.get(name)
+            if delayed_message_time:
+                if time.time() - delayed_message_time < self._retry_delay:
+                    logging.debug("Skipping collecting %s", name)
+                    continue
+                else:
+                    del self._delayed_messages[name]
             try:
                 message = handler.get_message_dict()
             except Exception as err:
-                name = handler.message_name
                 logging.error("the following error occurred while collecting %s: %s", name, err)
+                self._delayed_messages[name] = time.time()
+                logging.info("Will retry collecting %s in %d seconds", name, self._retry_delay)
                 message = None
 
             if message is not None:
@@ -597,7 +596,14 @@ def init(conf : dict) -> bool:
     interval = to_int(interval)
     if isinstance(interval, int) and interval > 0:
         global CLIENT
-        CLIENT = BluemanCollector(interval, uuid, conf)
+
+        # set retry delay
+        retry_delay = conf.get("retry-delay", 60 * 60)  # 1 hour as seconds
+        retry_delay = to_int(retry_delay)
+        if retry_delay < 0:
+            retry_delay = 60 * 60
+            logging.warning("invalid retry delay value, using default %d", retry_delay)
+        CLIENT = BluemanCollector(interval, uuid, conf, retry_delay)
         res = True
     else:
         logging.error("invalid interval value")
