@@ -24,9 +24,13 @@ FULL_TRANSFORM = 0
 PARTIAL_TRANSFORM = 1
 SCRIPT_TAG = "[collect_host_inventory]"
 LEGACY_MODE = False
+TIMEOUT = 30
+TIMEOUT_RETRY_DELAY = 60*60
 DEVICE_TYPE = "device_type"
 
 LOG_LEVEL_OPTION = "log-level"
+LEGACY_MODE_OPTION = "legacy-mode"
+TIMEOUT_RETRY_DELAY_OPTION = "timeout-retry-delay"
 
 # collector sampling interval
 NETWORK_INTERFACES_SAMPLING_OPTION = "network-interfaces-sampling-interval"
@@ -58,9 +62,49 @@ SECTION_TO_COLLECTOR = {}
 SCRIPT_START_TIME = time.time()
 LOGGER = logging.getLogger('collect_host_inventory')
 
+def arg_to_bool(arg):
+    ret = False
+    if isinstance(arg, str):
+        arg = arg.lower()
+        ret = arg in ("t", "true", "1")
+    elif isinstance(arg, bool):
+        ret = arg
+    elif arg:
+        ret = True
+    return ret
+
+def bool_from_dict(d, arg, default):
+    val = d.get(arg, default)
+    ret = arg_to_bool(val)
+    return ret
+
+def int_from_dict(d, arg, default):
+    val = d.get(arg, default)
+    if isinstance(val, str):
+        try:
+            val = int(val)
+        except Exception:
+            val = default
+    return val
 
 def get_command_output(command, check_rc=True):
-    output = subprocess.run(command, shell=True, capture_output=True, text=True)
+    if not hasattr(get_command_output, 'delayed_commands'):
+        get_command_output.delayed_commands = {}
+
+    global TIMEOUT, TIMEOUT_RETRY_DELAY
+
+    delayed_command = get_command_output.delayed_commands.get(command)
+    if delayed_command:
+        if time.time() - delayed_command < TIMEOUT_RETRY_DELAY:
+            return None
+        else:
+            del get_command_output.delayed_commands[command]
+    try:
+        output = subprocess.run(command, shell=True, capture_output=True, text=True, timeout=TIMEOUT)
+    except subprocess.TimeoutExpired:
+        LOGGER.error("got timeout for command %s, will not run it for the next %d seconds", command, TIMEOUT_RETRY_DELAY)
+        get_command_output.delayed_commands[command] = time.time()
+        return None
     if check_rc:
         try:
             output.check_returncode()
@@ -84,7 +128,6 @@ def get_hostname():
         hostname = NA
     return hostname.strip()
 
-
 class GeneralDataCollector():
     """
     Simple utility collector to pass general metadata for all messages.
@@ -105,6 +148,8 @@ class GeneralDataCollector():
             "timestamp": now,
             "hostname": cls.hostname,
         }
+        if LEGACY_MODE:
+            gm[DEVICE_TYPE] = cls.device_type
         return gm
 
     @classmethod
@@ -113,8 +158,6 @@ class GeneralDataCollector():
         metadata for message headers
         """
         id_key = "hostname"
-        if LEGACY_MODE:
-            id_key = "aid"
         return {
             # "aid" is an identifier index. here using machine hostname
             id_key: cls.hostname,
@@ -203,8 +246,6 @@ class TimerCollector:
                 else:
                     metadata[TRANS_MODE] = FULL_TRANSFORM
                     metadata[MESSAGE_TYPE] = section_name
-                    if isinstance(msg, dict) and LEGACY_MODE:
-                        msg[MESSAGE_TYPE] = section_name
                 if msg:
                     data = metadata
                     if isinstance(msg, list):
@@ -279,7 +320,7 @@ class IbDevicesCollector(TimerCollector):
     def _collect(self, gm):
         ip_addr_info = self.get_ibstat_info()
         if not ip_addr_info:
-            LOGGER.error("cannot collect ibstat ")
+            LOGGER.error("cannot collect ibstat")
             return {}
         data = gm
         data.update({"devices": ip_addr_info})
@@ -327,10 +368,10 @@ class NetworkInterfacesCollector(TimerCollector):
     def _collect(self, gm):
         ip_addr_info = self.get_ip_addr_show()
         if not ip_addr_info:
-            LOGGER.error("cannot collect network interface addresses ")
+            LOGGER.error("cannot collect network interface addresses")
             return {}
         data = gm
-        data.update({"interfces": ip_addr_info})
+        data.update({"interfaces": ip_addr_info})
         return data
 
     @classmethod
@@ -353,10 +394,10 @@ class LshwNetworkCollector(TimerCollector):
     def _collect(self, gm):
         ip_addr_info = self.get_lshw_network()
         if not ip_addr_info:
-            LOGGER.error("cannot collect lshw network ")
+            LOGGER.error("cannot collect lshw network")
             return {}
         data = gm
-        data.update({"interfces": ip_addr_info})
+        data.update({"interfaces": ip_addr_info})
         return data
 
     @classmethod
@@ -371,17 +412,38 @@ class LSPCICollector(TimerCollector):
     """
     Collector for LSPCI info
     """
+    # example 
+    # LnkSta: Speed 16GT/s (ok), Width x16 (ok)
+    PCI_LINK_STATE_RE = re.compile(r'LnkSta:\s+Speed\s+(\S+)\s+\((\S+)\),\s+Width\s+(\S+)\s+\((\S+)\)')
+    # example 
+    # [SN] Serial number: MT2116T00290
+    PCI_SERIAL_NUMBER_RE = re.compile(r'Serial number:\s+(\S+)')
+    # example
+    # Device: 0000:e2:00.0
+    # Device: MT28908 Family [ConnectX-6]
+    PCI_DEVICE_RE = re.compile(r"Device\:\s+(\S+.*)")
+    # example
+    # Class:  Infiniband controller
+    PCI_CLASS_RE = re.compile(r"Class\:\s+(\S+.*)")
+    # example
+    # Vendor: Mellanox Technologies
+    PCI_VENDOR_RE = re.compile(r"Vendor\:\s+(\S+.*)")
+    # example
+    # 0000:81:00.0 Infiniband controller: Mellanox Technologies MT28908 Family [ConnectX-6]
+    PCI_FULL_BUS_ADDRESS_RE = re.compile(r"([0-9a-f]{4}:[0-9a-f]{2}:[0-9a-f]{2}.[0-9a-f]+)\s*")
 
     def __init__(self, config):
-        sampling_interval = config.get(LSPCI_SAMPLING_OPTION, "3h")
+        sampling_interval = config.get(LSPCI_SAMPLING_OPTION, "1d")
         super().__init__(sampling_interval)
 
     def _collect(self, gm):
-        lspci_info = self.get_lspci_info()
-        if not lspci_info['bus_info']:
-            LOGGER.error("cannot collect lspci info ")
+        data = gm
+        bus_info = self.get_bus_info()
+        if not bus_info:
+            LOGGER.error("cannot collect lspci info")
             return {}
-        return lspci_info
+        data["bus_info"] = bus_info
+        return data
 
     @classmethod
     def get_hca_buses(cls):
@@ -389,26 +451,127 @@ class LSPCICollector(TimerCollector):
         cmd = 'lspci -D |grep -i Mellanox | egrep -i "(Ethernet|Infiniband|Network)" | grep -iv Virtual'
         res = get_command_output(command=cmd)
         if res is not None:
-            match = re.findall(r'([0-9a-f]{4}:[0-9a-f]{2}:[0-9a-f]{2}.[0-9a-f]+)\s+', res)
+            match = cls.PCI_FULL_BUS_ADDRESS_RE.findall(res)
             for bus in match:
                 port_buses.append(bus)
         return port_buses
 
     @classmethod
-    def get_lspci_info(cls):
-        data = {}
-        data.update({'bus_info': []})
-        port_buses = cls.get_hca_buses()
-        for bus in port_buses:
-            serial_number = ''
-            res = get_command_output(command=f'lspci -vvv -s {bus} |grep -i "Serial number"')
-            if res is not None:
-                match = re.search(r'Serial number:\s+(\S+)', res)
+    def get_hca_buses_description(cls, port_buses):
+        cmd = 'lspci -D -m -vvv -d 15b3:'
+        # output example
+        #
+        # Device: 0000:81:00.0
+        # Class:  Infiniband controller
+        # Vendor: Mellanox Technologies
+        # Device: MT28908 Family [ConnectX-6]
+        # SVendor:        Mellanox Technologies
+        # SDevice:        MT28908 Family [ConnectX-6]
+        # NUMANode:       1
+        # 
+        # Device: 0000:81:00.1
+        # Class:  Infiniband controller
+        # Vendor: Mellanox Technologies
+        # Device: MT28908 Family [ConnectX-6]
+        # SVendor:        Mellanox Technologies
+        # SDevice:        MT28908 Family [ConnectX-6]
+        # NUMANode:       1
+        description_lines_per_device = []
+        res = get_command_output(command=cmd)
+        if res is not None:
+            lines = res.splitlines()
+            
+            device_lines = []
+            for line in lines:
+                if line:
+                    # separation between paragraphs
+                    device_lines.append(line)
+                else:
+                    if device_lines:
+                        description_lines_per_device.append(device_lines)
+                    device_lines = []
+            if device_lines:
+                description_lines_per_device.append(device_lines)
+        description = {}
+        for lines in description_lines_per_device:
+            bus_description = {}
+            current_bus = None
+            for line in lines:
+                if len(bus_description) == 3:
+                    break
+                match = cls.PCI_DEVICE_RE.match(line)
                 if match:
-                    serial_number = match.group(1)
-            data['bus_info'].append({bus: {'serial_number': serial_number}})
-        return data
+                    first = match.group(1)
+                    match = cls.PCI_FULL_BUS_ADDRESS_RE.match(first)
+                    if match:
+                        first = match.group(1)
+                        current_bus = first.strip()
+                    else:
+                        # is device name and not device number
+                        bus_description["device"] = first.strip()
+                    continue
+                match = cls.PCI_CLASS_RE.match(line)
+                if match:
+                    class_name = match.group(1)
+                    bus_description["class"] = class_name.strip()
+                    continue
+                match = cls.PCI_VENDOR_RE.match(line)
+                if match:
+                    vendor = match.group(1)
+                    bus_description["vendor"] = vendor.strip()
+                    continue
+            if current_bus in port_buses:
+                description[current_bus] = bus_description
+        return description
 
+    @classmethod
+    def _parse_pci_serial_number(cls, output : str):
+        serial_number = NA
+        match = re.search(cls.PCI_SERIAL_NUMBER_RE, output)
+        if match:
+            serial_number = match.group(1)
+        return serial_number
+
+    @classmethod
+    def _parse_pci_link_state(cls, output : str):
+        link_state = NA
+        match = cls.PCI_LINK_STATE_RE.search(output)
+        if match is not None:
+            link_state = {
+                "speed_value": match.group(1),
+                "speed_status": match.group(2),
+                "width_value": match.group(3),
+                "width_status": match.group(4),
+            }
+        return link_state
+
+    @classmethod
+    def _parse_pci_description(cls, bus : str, output : str):
+        description : str = NA
+        if output:
+            end = output.find("\n")
+            if end > 0:
+                line = output[:end].strip()
+                # description starts at the second word
+                # example:
+                # "81:00.0 Infiniband controller: Mellanox Technologies MT28908 Family [ConnectX-6]"
+                first_space = output.find(" ")
+                if first_space > 0:
+                    description = line[first_space + 1 : ].strip()
+        return description
+
+    @classmethod
+    def get_bus_info(cls):
+        data = []
+        port_buses = cls.get_hca_buses()
+        buses_description = cls.get_hca_buses_description(port_buses)
+        for bus, info in buses_description.items():
+            res = get_command_output(command=f'lspci -vvv -s {bus}')
+            if res is not None:
+                info["serial_number"] = cls._parse_pci_serial_number(res)
+                info["link_state"] = cls._parse_pci_link_state(res)
+            data.append({bus: info})
+        return data
 
 class EthtoolCollector(TimerCollector):
     """
@@ -420,16 +583,17 @@ class EthtoolCollector(TimerCollector):
         super().__init__(sampling_interval)
 
     def _collect(self, gm):
-        ethtool_info = self.get_ethtool_info()
-        if not ethtool_info['standard_info']:
-            LOGGER.error("cannot collect ethtool info ")
+        data = gm
+        standard_info = self.get_standard_info()
+        if not standard_info:
+            LOGGER.error("cannot collect ethtool info")
             return {}
-        return ethtool_info
+        data["standard_info"] = standard_info
+        return data
 
     @classmethod
-    def get_ethtool_info(cls):
-        data = {}
-        data.update({'standard_info': []})
+    def get_standard_info(cls):
+        data = []
         bus_interface_dict = UtilMethods.get_map_bus_interface()
         for interface in bus_interface_dict.values():
             if interface:
@@ -441,14 +605,22 @@ class EthtoolCollector(TimerCollector):
                     match = re.search(r'Port:\s+(.*)', res)
                     if match:
                         connector_type = match.group(1)
-                data['standard_info'].append({interface: {'connector_type': connector_type}})
+                data.append({interface: {'connector_type': connector_type}})
         return data
 
 
-class RoCECollector(TimerCollector):
+class RoCEConfCollector(TimerCollector):
     """
     Collector for RoCE config info
     """
+    ROCE_ACCL_KEY_VAL_RE = re.compile(r"^(\S*)\s*\|\s*(\S+)\s*", re.MULTILINE)
+    MLNX_QOS_PTS_RE = re.compile(r"Priority trust state\:\s*(\S+)")
+    MLNX_QOS_DSCP2PRIO_RE = re.compile(r"dscp2prio mapping\:\s*")
+    MLNX_QOS_DSCP2PRIO_ENTRY_RE = re.compile(r"\s*prio:(\d+)\s*dscp\:((\d+,)+)")
+    MLNX_QOS_PFC_RE = re.compile(r"PFC configuration\:\s*")
+    MLNX_QOS_PFC_PRIORITY_RE = re.compile(r"\s*priority\s*((\d+\s*)+)")
+    MLNX_QOS_PFC_ENABLED_RE = re.compile(r"\s*enabled\s*((\d+\s*)+)")
+    MLNX_QOS_PFC_BUFFER_RE = re.compile(r"\s*buffer\s*((\d+\s*)+)")
 
     def __init__(self, config):
         sampling_interval = config.get(ROCE_SAMPLING_OPTION, "1h")
@@ -457,7 +629,7 @@ class RoCECollector(TimerCollector):
     def _collect(self, gm):
         roce_info = self.get_roce_config_info()
         if not roce_info:
-            LOGGER.error("cannot collect roce config info ")
+            LOGGER.error("cannot collect roce config info")
             return {}
         data = gm
         data["roce_info"] = roce_info
@@ -498,10 +670,109 @@ class RoCECollector(TimerCollector):
                     dscp_info = cls.get_dscp_traffic_class(rdma_device=rdma_device)
                     port_data_info.update({"DSCP": {"traffic_class": dscp_info}})
 
+                    # ROCE_ACCL
+                    roce_accl = cls.get_roce_accl(bus=bus)
+                    port_data_info.update({"ROCE_ACCL": roce_accl})
+
+                    # mlnx_qos
+                    pfc, dscp2prio, pts = cls.get_mlnx_qos_entries(interface)
+                    port_data_info["DSCP"].update({"dscp2prio": dscp2prio})
+                    port_data_info.update({"Priority_trust_state": pts})
+                    port_data_info.update({"PFC": pfc})
+
                     port_data_info.update(ident_data)
             data.append(port_data_info)
         return data
 
+    @classmethod
+    def get_mlnx_qos_entries(cls, interface):
+        pfc, dscp2prio, pts = {}, {}, NA
+        if not interface:
+            return pfc, dscp2prio, pts
+        res = get_command_output(command=f"mlnx_qos -i {interface}")
+        if res is not None:
+            in_dscp2prio = False
+            in_pfc = False
+            pfc_prios = []
+            pfc_enabled = []
+            pfc_buffers = []
+            lines = res.splitlines()
+            for line in lines:
+                if in_dscp2prio:
+                    match = cls.MLNX_QOS_DSCP2PRIO_ENTRY_RE.match(line)
+                    if match:
+                        prio = match.group(1)
+                        dscp = match.group(2)
+                        dscp = dscp.split(",")
+                        vals = []
+                        for v in dscp:
+                            v = v.strip()
+                            if v:
+                                vals.append(v)
+                        dscp2prio[prio] = vals
+                        continue
+                    in_dscp2prio = False
+                if in_pfc:
+                    match = cls.MLNX_QOS_PFC_PRIORITY_RE.match(line)
+                    if match:
+                        prios = match.group(1)
+                        prios = prios.split(" ")
+                        for p in prios:
+                            p = p.strip()
+                            if p:
+                                pfc_prios.append(p)
+                        continue
+                    match = cls.MLNX_QOS_PFC_ENABLED_RE.match(line)
+                    if match:
+                        enabled = match.group(1)
+                        enabled = enabled.split(" ")
+                        for e in enabled:
+                            e = e.strip()
+                            if e:
+                                pfc_enabled.append(e)
+                        continue
+                    match = cls.MLNX_QOS_PFC_BUFFER_RE.match(line)
+                    if match:
+                        buffers = match.group(1)
+                        buffers = buffers.split(" ")
+                        for b in buffers:
+                            b = b.strip()
+                            if b:
+                                pfc_buffers.append(b)
+                        continue
+                    in_pfc = False
+                match = cls.MLNX_QOS_PFC_RE.match(line)
+                if match:
+                    in_pfc = True
+                match = cls.MLNX_QOS_PTS_RE.match(line)
+                if match:
+                    pts = match.group(1).strip()
+                    continue
+                match = cls.MLNX_QOS_DSCP2PRIO_RE.match(line)
+                if match:
+                    in_dscp2prio = True
+                    continue
+            if pfc_prios and len(pfc_prios) == len(pfc_enabled) == len(pfc_buffers):
+                for i, prio in enumerate(pfc_prios):
+                    pfc[prio] = {
+                        "enabled": pfc_enabled[i],
+                        "buffer": pfc_buffers[i],
+                    }
+        return pfc, dscp2prio, pts
+
+    @classmethod
+    def get_roce_accl(cls, bus):
+        roce_accl = {}
+        res = get_command_output(command=f"mlxreg -d {bus} --reg_name ROCE_ACCL --get")
+        if res is not None:
+            matches = cls.ROCE_ACCL_KEY_VAL_RE.findall(res)
+            for match in matches:
+                key = match[0]
+                value = match[1]
+                roce_accl[key] = value
+        return roce_accl
+
+
     @classmethod
     def get_roce_mode(cls, rdma_device):
         mode = ''
@@ -598,7 +869,7 @@ class NodeCollector(TimerCollector):
 
     @classmethod
     def _get_ip(cls, data):
-        key = "ip"
+        key = "ipv4"
         txt = get_command_output(command="hostname -i")
         data[key] = NA
         if txt is not None:
@@ -637,7 +908,7 @@ def set_section_to_collector(config):
         "LSHWNetwork": LshwNetworkCollector(config),
         "LSPCI": LSPCICollector(config),
         "Ethtool": EthtoolCollector(config),
-        "RoCE": RoCECollector(config),
+        "RoCEConf": RoCEConfCollector(config),
         "Node": NodeCollector(config),
     }
     enabled_components = config.get("enabled", "")
@@ -670,6 +941,9 @@ def set_logging(config):
 
 
 def init(config):
+    global LEGACY_MODE, TIMEOUT_RETRY_DELAY
+    LEGACY_MODE = bool_from_dict(config, LEGACY_MODE_OPTION, LEGACY_MODE)
+    TIMEOUT_RETRY_DELAY = int_from_dict(config, TIMEOUT_RETRY_DELAY_OPTION, TIMEOUT_RETRY_DELAY)
     set_logging(config)
     GeneralDataCollector.hostname = get_hostname()
     set_section_to_collector(config)
