@@ -1,8 +1,15 @@
 /*
-* Copyright (C) Mellanox Technologies Ltd. 2018.  ALL RIGHTS RESERVED.
-*
-* See file LICENSE for terms.
-*/
+ * Copyright © 2019-2024 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+ *
+ * This software product is a proprietary product of Nvidia Corporation and its affiliates
+ * (the "Company") and all right, title, and interest in and to the software
+ * product, including all associated intellectual property rights, are and
+ * shall remain exclusively with the Company.
+ *
+ * This software product is governed by the End User License Agreement
+ * provided with the software product.
+ */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <assert.h>
@@ -14,6 +21,10 @@
 #include <string.h>
 #include <unistd.h>
 
+/// Include the following macro to allow proper logger usage between several providers.
+/// It is crucial when providers run as part of grpc / plugin runner / clx_api.
+#define DEFINE_CLX_REGISTER_LOGGER
+
 #include "common/logger.h"
 #include "common/status.h"
 #include "counters/counter_data.h"
@@ -23,7 +34,7 @@
 #define EXAMPLE_STR_LEN 32
 // General explanation
 //
-// Counter porvider contains one or several components and implements counter group callbacks.
+// Counter provider contains one or several components and implements counter group callbacks.
 //
 // - Components:
 //     Provider contains all available components. Components may be enabled/disabled via `clx_config.ini`.
@@ -118,7 +129,7 @@ static bool init_example_counters(clx_provider_t* provider, clx_component_t* com
     }
     comp->counters = counters;
 
-    // Allocate fisrt counter object with numberic int64_t value type
+    // Allocate first counter object with numeric int64_t value type
     clx_counter_info_t* counter_1 = (clx_counter_info_t*) calloc(1, sizeof(clx_counter_info_t));
     if (counter_1 == NULL) {
         log_error("Failed to allocate counter object: %s", strerror(errno));
@@ -181,7 +192,7 @@ static bool init_example_counters(clx_provider_t* provider, clx_component_t* com
 
 static bool init_example_component(clx_provider_t* provider) {
     ///
-    /// Initialize counter porvider component
+    /// Initialize counter provider component
     ///
 
     /// 1. Allocate component
@@ -251,8 +262,8 @@ static bool example_init_provider(clx_provider_t* provider, const struct clx_opt
     ///
     /// Implement `.initialize` provider callback.
     /// Allocate provider-specific structures and create of one or several components.
-    /// Provider components are accosiated with counter groups.
-    /// Components can be turned on/off from user defuned options (app_opts).
+    /// Provider components are associated with counter groups.
+    /// Components can be turned on/off from user defined options (app_opts).
     ///
 
     provider->extra = (example_provider_context_t*) calloc(1, sizeof(example_provider_context_t));
@@ -278,7 +289,7 @@ static void example_provider_handle_command(clx_provider_t* provider,
     assert(request  != NULL);
     assert(response != NULL);
 
-    /// Get command from request and prepare responce with `cmd_set_string_by_key`
+    /// Get command from request and prepare response with `cmd_set_string_by_key`
 
     /// Option 1. Accept command with command_key and boolean command_new_val to set it
     ///           to example_boolean_param if new val differs from the old one.
@@ -352,7 +363,7 @@ static bool example_add_counter_to_group(clx_counter_group_t* group,
                                        const clx_counter_info_t* counter) {
     ///
     /// This callback does the following:
-    ///    -# coppies mapping idx_to_ptr from component to the counter group,
+    ///    -# copies mapping idx_to_ptr from component to the counter group,
     ///    -# updates number of group counters
     /// Actual counter info is copied outside of the provider.
     /// Note that if clx_config.ini name match if enabled, number of counters in component and group mi
@@ -453,29 +464,30 @@ static bool example_read_counter_group_values(clx_counter_group_t* group,
  * Although most of interaction with counter group is made by clx collector, these callbacks can be reimplemented
  * if they interact with new provider-specific structures.
 */
-static clx_provider_t example_provider = {
-    .name               = "example",
-    .description        = "Counter provider example",
-    .version            = 0x010000,
-    .num_components     = 0,
-    .components         = NULL,                                /// will be allocated in .initialize callback
-    .extra              = NULL,                                /// use for provider specific fields
-    .initialize         = &example_init_provider,              /// implement (alloc structs)
-    .finalize           = &example_finalize_provider,          /// implement (free structs)
-    .command            = &example_provider_handle_command,    /// optional
-
-     // group callbacks
-    .group.create       = &example_create_counter_group,       /// ignore
-    .group.destroy      = &example_destroy_counter_group,      /// ignore
-    .group.add_counter  = &example_add_counter_to_group,       /// implement
-    .group.start        = &example_start_counting_group,       /// ignore
-    .group.stop         = &example_stop_counting_group,        /// ignore
-    .group.read         = &example_read_counter_group_values,  /// implement
-};
-
+static clx_provider_t example_provider;
 
 clx_provider_t* construct_counter_provider(const struct clx_options_t* app_opts) {
     log_debug("Construct example counter provider");
+    example_provider.name               = "example_counters";
+    example_provider.description        = "Counter provider example";
+    example_provider.version            = 0x010000;
+    example_provider.num_components     = 0;
+    example_provider.components         = NULL;                                /// will be allocated in .initialize callback
+    example_provider.extra              = NULL;                                /// use for provider specific fields
+    example_provider.initialize         = &example_init_provider;              /// implement (alloc structs)
+    example_provider.finalize           = &example_finalize_provider;          /// implement (free structs)
+    example_provider.command            = &example_provider_handle_command;    /// optional
+
+     // group callbacks
+    example_provider.group.create       = &example_create_counter_group;       /// ignore
+    example_provider.group.destroy      = &example_destroy_counter_group;      /// ignore
+    example_provider.group.add_counter  = &example_add_counter_to_group;       /// implement
+    example_provider.group.start        = &example_start_counting_group;       /// ignore
+    example_provider.group.stop         = &example_stop_counting_group;        /// ignore
+    example_provider.group.read         = &example_read_counter_group_values;  /// implement
+
+
+
     example_init_provider(&example_provider, app_opts);
     return &example_provider;
 }
