@@ -1,8 +1,15 @@
 /*
-* Copyright (C) Mellanox Technologies Ltd. 2019.  ALL RIGHTS RESERVED.
-*
-* See file LICENSE for terms.
-*/
+ * Copyright © 2019-2024 NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+ *
+ * This software product is a proprietary product of Nvidia Corporation and its affiliates
+ * (the "Company") and all right, title, and interest in and to the software
+ * product, including all associated intellectual property rights, are and
+ * shall remain exclusively with the Company.
+ *
+ * This software product is governed by the End User License Agreement
+ * provided with the software product.
+ */
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -13,6 +20,10 @@
 #include <stdbool.h>
 #include <unistd.h>
 
+/// Include the following macro to allow proper logger usage between several providers.
+/// It is crucial when providers run as part of grpc / plugin runner / clx_api.
+#define DEFINE_CLX_REGISTER_LOGGER
+
 #include "common/logger.h"
 #include "common/dict.h"
 #include "data/source.h"
@@ -38,7 +49,9 @@ typedef struct __attribute__((packed)) example_event_t {
 struct example_event_provider_t;
 
 // callback for source processor
-typedef int (*example_event_source_processor_t)(struct example_event_provider_t* example_provider, clx_data_serializer_t* ds);
+typedef clx_progress_status_t
+        (*example_event_source_processor_t)(struct example_event_provider_t* example_provider,
+                                            clx_data_serializer_t* ds);
 
 ///
 /// Customize provider on the basis of the current example:
@@ -85,12 +98,12 @@ bool example_event_provider_initialize(clx_event_provider_t* provider, clx_type_
     /// Provider data sources will use one schema, but may use same or different types of the schema.
     ///
 
-    example_event_provider_t* test_provider = (example_event_provider_t*) provider;
+    example_event_provider_t* p = (example_event_provider_t*) provider;
 
 
     /// 1. create schema as a part of type_system *ts* and save schema_index to provider
     clx_schema_t* schema = clx_type_system_add_schema(ts, provider->name, provider->version,
-                                                      &(test_provider->schema_index));
+                                                      &(p->schema_index));
     if (schema == NULL) {
         return false;
     }
@@ -147,7 +160,7 @@ bool example_event_provider_initialize(clx_event_provider_t* provider, clx_type_
 
     if (CLX_TS_OK != clx_schema_add_type(schema, "example_event",
                                          example_event_fields, nof_array_elements(example_event_fields),
-                                         &(test_provider->type_index))) {
+                                         &(p->type_index))) {
         return false;
     }
 
@@ -162,7 +175,7 @@ clx_source_array_t* example_event_provider_get_sources(struct clx_event_provider
     ///
     /// Implement `example_event_provider_get_sources()` to create sources array and to return them clx.
     /// `.progress` callback will deliver data from the provider to clx on source-basis.
-    /// Source is a pair of two strings "id" (e.g. node/swith name) and "tag" (used to create bin-file name, e.g. tag_1234556.bin).
+    /// Source is a pair of two strings "id" (e.g. node/switch name) and "tag" (used to create bin-file name, e.g. tag_1234556.bin).
     /// Consider using:
     ///     - clx_source_t* source = `clx_create_default_source("id_name", "tag_name");
     ///     - clx_source_array_append(sources, source);             to append source to sources array
@@ -220,7 +233,7 @@ bool example_event_provider_start(clx_event_provider_t* provider) {
 
 static void example_event_provider_generate_events(example_event_provider_t* example_provider) {
     ///
-    /// `example_event_provider_generate_events()` is the helper for retreiving the data
+    /// `example_event_provider_generate_events()` is the helper for retrieving the data
     ///
     memset(example_provider->events_cache, 0, sizeof(example_provider->events_cache));
 
@@ -307,18 +320,18 @@ clx_progress_status_t example_event_provider_progress(clx_event_provider_t* prov
                                                    clx_data_serializer_t* ds) {
     ///
     /// Implement `example_event_provider_progress` function to allow clx to get the cached data from the provider.
-    /// clx writes data into its buffer and syncronizes it (writes/exports data) when buffer is full or on force_sync request.
+    /// clx writes data into its buffer and synchronizes it (writes/exports data) when buffer is full or on force_sync request.
     /// To write the data use one of 2 options:
     ///    - int num_written_events = clx_data_serializer_write_events(ds, schema_index, type_index, cashed_data, pending_events);
     ///    - example_event_t* example_event =
     ///                   (example_event_t*) clx_data_serializer_get_events_buffer(ds, schema_index, type_index, 1, &buffer_size);
     ///
     /// progress callback should return:
-    ///    - CLX_PROGRESS_SUCCESS     if all pending events were written to clx buffer successfuly.
+    ///    - CLX_PROGRESS_SUCCESS     if all pending events were written to clx buffer successfully.
     ///    - CLX_PROGRESS_MORE_DATA   if clx cannot receive all events. clx will call `.progress`
     ///                               callback again instantly to get the rest of the data.
     ///    - CLX_PROGRESS_FORCE_SYNC  to tell clx to export/write data to disk immediately
-    ///    - CLX_PROGRESS_ERROR       if cannot get clx buffer or on the other errorneous scenarios
+    ///    - CLX_PROGRESS_ERROR       if cannot get clx buffer or on the other erroneous scenarios
     ///
     /// see impl functions `example_event_provider_progress_source_[0|1]` for more examples
     assert(source_index < NUM_EVENT_SOURCES);
@@ -332,7 +345,7 @@ bool example_event_provider_stop(clx_event_provider_t* provider) {
     ///
     /// Implement `example_event_provider_stop()` by picking up one of 2 options:
     ///     - `return true` if the provider does not use the separate thread
-    ///     - finilize the pthread job
+    ///     - finalize the pthread job
     ///
 
     return true;
@@ -379,7 +392,7 @@ void example_event_provider_command(struct clx_event_provider_t* provider,
     assert(request  != NULL);
     assert(response != NULL);
 
-    /// Get command from request and prepare responce with `cmd_set_string_by_key`
+    /// Get command from request and prepare response with `cmd_set_string_by_key`
 
     /// Option 1. Accept command with command_key and boolean command_new_val to set it
     ///           to example_boolean_param if new val differs from the old one.
@@ -423,50 +436,48 @@ void example_event_provider_command(struct clx_event_provider_t* provider,
 
 
 /// Initialize the provider according to its structure and return it in `construct_event_provider()`
-static example_event_provider_t example_event_provider = {
-    .base = {
-        .name               = "example_events",
-        .description        = "Description of the provider",
-        .version            = {{1, 0, 0}},
-
-        // implement the following callbacks:
-        // .initialize, .start and .get sources will be called at the beginning one after another
-        .initialize         = &example_event_provider_initialize,
-        .start              = &example_event_provider_start,
-        .get_sources        = &example_event_provider_get_sources,
-
-        // progress function is the main routine
-        .progress           = &example_event_provider_progress,
-
-        // finalization steps
-        .stop               = &example_event_provider_stop,
-        .finalize           = &example_event_provider_finalize,
-
-        // keep this callback with the default values
-        .get_source_config  = &example_event_provider_get_source_config,
-        .command            = &example_event_provider_command,
-    },
-    .type_index = CLX_UNDEFINED_TYPE,
-    .pending_events         = 0,
-    .processors = {
-        &example_event_provider_progress_source_0,
-        &example_event_provider_progress_source_1,
-    },
-    .val_for_source_0       = 0,
-    .val_for_source_1       = 0,
-    .example_boolean_param  = 0,
-};
-
+static example_event_provider_t example_event_provider;
 
 clx_event_provider_t* construct_event_provider(const clx_dict_t* opts) {
     /// `construct_event_provider()` function is mandatory for event providers.
 
+    example_event_provider.base.name               = "example_events";
+    example_event_provider.base.description        = "Description of the provider";
+    example_event_provider.base.version            = (clx_version_t){{1, 0, 0}};
+
+    // implement the following callbacks:
+    // .initialize, .start and .get sources will be called at the beginning one after another
+    example_event_provider.base.initialize         = &example_event_provider_initialize;
+    example_event_provider.base.start              = &example_event_provider_start;
+    example_event_provider.base.get_sources        = &example_event_provider_get_sources;
+
+    // progress function is the main routine
+    example_event_provider.base.progress           = &example_event_provider_progress;
+
+    // finalization steps
+    example_event_provider.base.stop               = &example_event_provider_stop;
+    example_event_provider.base.finalize           = &example_event_provider_finalize;
+
+    // keep this callback with the default values
+    example_event_provider.base.get_source_config  = &example_event_provider_get_source_config;
+    example_event_provider.base.command            = &example_event_provider_command;
+
+    example_event_provider.type_index              = CLX_UNDEFINED_TYPE;
+    example_event_provider.pending_events          = 0;
+    example_event_provider.processors[0]           = &example_event_provider_progress_source_0;
+    example_event_provider.processors[1]           = &example_event_provider_progress_source_1;
+
+    example_event_provider.val_for_source_0       = 0;
+    example_event_provider.val_for_source_1       = 0;
+    example_event_provider.example_boolean_param  = 0;
+
     /// 1. Get custom options form app_opts which is parsed clx_config.ini file.
     /// clx_config.ini:
     /// custom_opt_1_key=val_1
     /// custom_opt_2_key=val_2
     ///
     /// if opts == NULL `clx_dict_get()` returns NULL
+
     example_event_provider.custom_option_1 = clx_dict_get(opts, "custom_opt_1_key");
     example_event_provider.custom_option_2 = clx_dict_get(opts, "custom_opt_2_key");
 
