@@ -372,6 +372,9 @@ class General(BaseModule):
             # # cat /etc/mlnx-release
             # DOCA_v1.1_BlueField_OS_Ubuntu_20.04-5.4.0-1013-bluefield-5.4-1.0.3.0-3.7.0.11805-1.signed-aarch64
             # the new convention: #DOCA_1.3.0_BSP_3.9.0_Ubuntu_20.04-6
+            # bf-bundle-2.7.0-31_24.04_ubuntu-22.04_prod
+            if 'bf-bundle-' in result.output:
+                match = re.search(r'bf-bundle-(\d+.\d+.\d+)', result.output, re.IGNORECASE)
             if 'DOCA_v' in result.output:
                 match = re.search(r'DOCA_v(\S+)_B',result.output,re.IGNORECASE)
             else:
@@ -396,7 +399,11 @@ class General(BaseModule):
             # DOCA_v1.1_BlueField_OS_Ubuntu_20.04-5.4.0-1013-bluefield-5.4-1.0.3.0-3.7.0.11805-1.signed-aarch64
             # the new convention: #DOCA_1.3.0_BSP_3.9.0_Ubuntu_20.04-6
             # DOCA_1.9.5_BSP_4.0.bu1_Ubuntu_22.04-69.20221213
-            if 'DOCA_v' in result.output:
+            # bf-bundle-2.7.0-31_24.04_ubuntu-22.04_prod
+            match = None
+            if 'bf-bundle' in result.output:
+                bluefield_os_version = result.output
+            elif 'DOCA_v' in result.output:
                 match = re.search(r'bluefield\S+-((\d+|\d+.){3}).\d{3,}-\d+.', result.output, re.IGNORECASE)
             else:
                 match = re.search(r'BSP_(\d+.[a-z,A-Z,0-9,.]+)_', result.output, re.IGNORECASE)
